/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* GNU AGPL v3 which accompanies
* this distribution, and is available at
* http://www.gnu.org/licenses/agpl.txt
*
* Contributors:
*
* Simon Dutkowski Fraunhofer FOKUS
*
* Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*/

package eu.DECIDEh2020.architect.cloudpatterns;

import org.apache.jena.query.*;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.riot.Lang;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.vocabulary.*;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

class PatternsImpl implements Patterns {

    private static Model patternsModel;

    PatternsImpl() {
        if (patternsModel == null) {
            patternsModel = ModelFactory.createDefaultModel();
            RDFDataMgr.read(patternsModel, PatternsImpl.class.getClassLoader().getResourceAsStream("patterns/decide-patterns.ttl"), Lang.TURTLE);
        }
    }

    @Override
    public Model getModel() {
        return patternsModel;
    }

    @Override
    public List<Resource> query(String queryString) {
        Query query = QueryFactory.create(queryString);
        return execute(query);
    }

    @Override
    public List<Resource> listPatterns() {
        return patternsModel.listResourcesWithProperty(RDF.type, DP.Pattern).toList();
    }

    @Override
    public List<Resource> getPatternsByCategory(Resource type) {
        Resource res = patternsModel.getResource(type.getURI());
        if (res != null && res.hasProperty(SKOS.inScheme, DECIDEPatterns.patternCategories)) {
            return patternsModel.listResourcesWithProperty(DCTerms.type, res).toList();
        } else {
            return null;
        }
    }

    @Override
    public Resource getPattern(String uri) {
        return patternsModel.getResource(uri);
    }

    @Override
    public InputStream getIcon(Resource icon) {
        String fileName = icon.getLocalName();
        return PatternsImpl.class.getClassLoader().getResourceAsStream("patterns/icons/" + fileName);
    }

    @Override
    public List<Resource> listNFRs() {
        return patternsModel.listResourcesWithProperty(RDF.type, DP.NFR).toList();
    }

    @Override
    public List<Resource> inferPatterns(List<Resource> nfrs) {
        // list all patterns that have an positive impact on these nfrs.
        // we should leave out fundamentals!
        StringBuffer buffer = new StringBuffer("select ?pattern where { ?pattern <http://decideh2020.eu/ns/patterns/hasPositiveImpactOn> ?nfr . filter ( ?nfr in (");
        nfrs.forEach(nfr -> {
            buffer.append("<");
            buffer.append(nfr.getURI());
            buffer.append(">, ");
        });
        buffer.setLength(buffer.length() - 2);
        buffer.append(") && not exists { ?pattern <http://purl.org/dc/terms/type> <http://decideh2020.eu/resources/patterncategories/fundamental> } ) }");
        Query query = QueryFactory.create(buffer.toString());
        return execute(query);
    }

    public List<Resource> inferBadPatterns(List<Resource> nfrs) {
        // list all patterns that have a negative impact on these nfrs.
        // we should probably NOT leave out fundamentals!
        StringBuffer buffer = new StringBuffer("select ?pattern where { ?pattern <http://decideh2020.eu/ns/patterns/hasNegativeImpactOn> ?nfr . filter ( ?nfr in (");
        nfrs.forEach(nfr -> {
            buffer.append("<");
            buffer.append(nfr.getURI());
            buffer.append(">, ");
        });
        buffer.setLength(buffer.length() - 2);
        buffer.append(") ) }");
        Query query = QueryFactory.create(buffer.toString());
        return execute(query);
    }

    private List<Resource> execute(Query query) {
        QueryExecution exec = QueryExecutionFactory.create(query, patternsModel);
        ResultSet result = exec.execSelect();
        List<Resource> patterns = new ArrayList<>();
        result.forEachRemaining(r -> patterns.add(r.getResource("pattern")));
        return patterns;
    }

}
