/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* GNU AGPL v3 which accompanies
* this distribution, and is available at
* http://www.gnu.org/licenses/agpl.txt
*
* Contributors:
*
* Simon Dutkowski Fraunhofer FOKUS
*
* Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*/

package eu.DECIDEh2020.architect.cloudpatterns.compendium;

import eu.DECIDEh2020.architect.cloudpatterns.compendium.service.CloudPatternsService;
import io.vertx.core.*;
import io.vertx.core.json.JsonArray;
import io.vertx.core.json.JsonObject;
import io.vertx.ext.dropwizard.MetricsService;
import io.vertx.ext.healthchecks.HealthCheckHandler;
import io.vertx.ext.healthchecks.Status;
import io.vertx.ext.web.Router;
import io.vertx.ext.web.handler.BodyHandler;
import io.vertx.ext.web.handler.StaticHandler;
import io.vertx.ext.web.handler.TemplateHandler;
import io.vertx.ext.web.templ.TemplateEngine;
import io.vertx.ext.web.templ.ThymeleafTemplateEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class Server extends AbstractVerticle {
    private Logger log = LoggerFactory.getLogger(getClass());

    public static void main(String[] args) {
        Launcher.executeCommand("run", Server.class.getName());
    }

    private CloudPatternsService patternsService;

    @Override
    public void start(Future<Void> startFuture) throws Exception {
        MetricsService metrics = MetricsService.create(vertx);

        vertx.deployVerticle(CloudPatternsCompendiumVerticle.class.getName(), new DeploymentOptions().setWorker(true), res -> {
            if (res.succeeded()) {
                patternsService = CloudPatternsService.createProxy(vertx, "cloudpatterns");

                Router router = Router.router(vertx);
                router.route().handler(BodyHandler.create());

                router.route("/").handler(StaticHandler.create());
                router.route("/openapi/*").handler(StaticHandler.create("openapi"));
                router.route("/resources/*").handler(StaticHandler.create("patterns"));

                HealthCheckHandler hch = HealthCheckHandler.create(vertx);
                hch.register("simple-ok", future -> future.complete(Status.OK()));
                hch.register("status", future -> future.complete(Status.OK(new JsonObject().put("build", 1))));
                router.get("/health*").handler(hch);

                router.get("/metrics").produces("application/json").handler(routingContext -> {
                    JsonObject m = metrics.getMetricsSnapshot(vertx);
                    if (m != null) {
                        routingContext.response().setStatusCode(200).end(m.encodePrettily());
                    } else {
                        routingContext.response().setStatusCode(503).end();
                    }
                });

                TemplateEngine engine = ThymeleafTemplateEngine.create();
                router.get("/templates/*").handler(TemplateHandler.create(engine));

                router.get("/nfrs").produces("application/json").handler(routingContext -> {
                    patternsService.listNFRs(reply -> {
                        if (reply.succeeded()) {
                            routingContext.response().setStatusCode(200);
                            routingContext.response().end(reply.result().encodePrettily());
                        } else {
                            routingContext.fail(500);
                        }
                    });
                });

                router.get("/patterns").produces("application/json").handler(routingContext -> {
                    patternsService.listPatterns(reply -> {
                        if (reply.succeeded()) {
                            routingContext.response().setStatusCode(200);
                            routingContext.response().end(reply.result().encodePrettily());
                        } else {
                            routingContext.fail(500);
                        }
                    });
                });

                router.get("/patterns/:pattern").produces("text/html").handler(routingContext -> {
                    String patternName = routingContext.request().getParam("pattern");
                    patternsService.getPattern(patternName, reply -> {
                        if (reply.succeeded()) {
                            routingContext.put("pattern", reply.result());
                            routingContext.reroute("/templates/pattern.html");
                        } else {
                            routingContext.response().setStatusCode(404);
                            routingContext.response().end("Pattern not found.");
                        }
                    });
                });

                router.get("/patterns/:pattern").produces("application/json").handler(routingContext -> {
                    String patternName = routingContext.request().getParam("pattern");
                    patternsService.getPattern(patternName, reply -> {
                        if (reply.succeeded()) {
                            routingContext.response().setStatusCode(200);
                            routingContext.response().end(reply.result().encodePrettily());
                        } else {
                            routingContext.response().setStatusCode(404);
                            routingContext.response().end("Pattern not found.");
                        }
                    });
                });

                router.get("/inferred/patterns").produces("application/json").handler(routingContext -> {
                    String param = routingContext.request().getParam("nfr");
                    if (param != null) {
                        String[] nfrs = param.split(",");
                        patternsService.inferPatterns(new JsonArray(Arrays.asList(nfrs)),reply -> {
                            if (reply.succeeded()) {
                                routingContext.response().setStatusCode(200);
                                routingContext.response().end(reply.result().encodePrettily());
                            } else {
                                routingContext.fail(500);
                            }
                        });
                    } else {
                        routingContext.fail(500);
                    }
                });

                vertx.createHttpServer().requestHandler(router::accept).listen(8080);

                startFuture.complete();
            } else {
                startFuture.fail(res.result());
            }
        });
    }

}
